package sun.security.provider;

import java.io.FileInputStream;
import java.security.PrivilegedExceptionAction;
import java.net.URL;
import java.io.InputStream;
import java.io.IOException;
import java.net.NetworkInterface;
import java.security.AccessController;
import java.util.Iterator;
import java.nio.file.DirectoryStream;
import java.util.Enumeration;
import java.util.Properties;
import java.nio.file.Path;
import java.util.Random;
import java.nio.file.Files;
import java.io.File;
import java.security.PrivilegedAction;
import java.security.NoSuchAlgorithmException;
import java.security.MessageDigest;
import sun.security.util.Debug;

abstract class SeedGenerator
{
    private static SeedGenerator instance;
    private static final Debug debug;
    
    SeedGenerator() {
    }
    
    public static void generateSeed(byte[] array) {
        SeedGenerator.instance.getSeedBytes(array);
    }
    
    abstract void getSeedBytes(byte[] p0);
    
    static byte[] getSystemEntropy() {
        final MessageDigest instance;
        try {
            instance = MessageDigest.getInstance("SHA");
        }
        catch (NoSuchAlgorithmException cause) {
            throw new InternalError("internal error: SHA-1 not available.", cause);
        }
        instance.update((byte)(int)System.currentTimeMillis());
        AccessController.doPrivileged(new PrivilegedAction<Void>() {
            @Override
            public Void run() {
                try {
                    Properties properties = System.getProperties();
                    Enumeration<Object> propertyNames = properties.propertyNames();
                    while (propertyNames.hasMoreElements()) {
                        String key = (String)propertyNames.nextElement();
                        instance.update(key.getBytes());
                        instance.update(properties.getProperty(key).getBytes());
                    }
                    SeedGenerator.access$000(instance);
                    File o = new File(properties.getProperty("java.io.tmpdir"));
                    int n = 0;
                    try (final DirectoryStream directoryStream = Files.newDirectoryStream(o.toPath())) {
                        Random o2 = new Random();
                        for (Path path : directoryStream) {
                            if (n < 512 || o2.nextBoolean()) {
                                instance.update(path.getFileName().toString().getBytes());
                            }
                            if (n++ > 1024) {
                                break;
                            }
                            continue;
                        }
                    }
                }
                catch (Exception ex) {
                    instance.update((byte)ex.hashCode());
                }
                Runtime runtime = Runtime.getRuntime();
                byte[] access$100 = SeedGenerator.access$100(runtime.totalMemory());
                instance.update(access$100, 0, access$100.length);
                byte[] access$101 = SeedGenerator.access$100(runtime.freeMemory());
                instance.update(access$101, 0, access$101.length);
                return null;
            }
        });
        return instance.digest();
    }
    
    private static void addNetworkAdapterInfo(MessageDigest messageDigest) {
        try {
            Enumeration<NetworkInterface> networkInterfaces = NetworkInterface.getNetworkInterfaces();
            while (networkInterfaces.hasMoreElements()) {
                NetworkInterface networkInterface = (NetworkInterface)networkInterfaces.nextElement();
                messageDigest.update(networkInterface.toString().getBytes());
                if (!networkInterface.isVirtual()) {
                    byte[] hardwareAddress = networkInterface.getHardwareAddress();
                    if (hardwareAddress != null) {
                        messageDigest.update(hardwareAddress);
                        break;
                    }
                    continue;
                }
                else {
                    continue;
                }
            }
        }
        catch (Exception ex) {}
    }
    
    private static byte[] longToByteArray(long n) {
        byte[] array = new byte[8];
        for (int i = 0; i < 8; ++i) {
            array[i] = (byte)(int)n;
            n >>= 8;
        }
        return array;
    }
    
    static void access$000(MessageDigest messageDigest) {
        addNetworkAdapterInfo(messageDigest);
    }
    
    static byte[] access$100(long n) {
        return longToByteArray(n);
    }
    
    static {
        debug = Debug.getInstance("provider");
        String seedSource = SunEntries.getSeedSource();
        Label_0209: {
            if (!seedSource.equals("file:/dev/random") && !seedSource.equals("file:/dev/urandom")) {
                if (seedSource.length() != 0) {
                    try {
                        SeedGenerator.instance = new URLSeedGenerator(seedSource);
                        if (SeedGenerator.debug != null) {
                            SeedGenerator.debug.println("Using URL seed generator reading from " + seedSource);
                        }
                    }
                    catch (IOException ex) {
                        if (SeedGenerator.debug != null) {
                            SeedGenerator.debug.println("Failed to create seed generator with " + seedSource + ": " + ex.toString());
                        }
                    }
                    break Label_0209;
                }
                break Label_0209;
            }
            try {
                SeedGenerator.instance = new NativeSeedGenerator(seedSource);
                if (SeedGenerator.debug != null) {
                    SeedGenerator.debug.println("Using operating system seed generator" + seedSource);
                }
            }
            catch (IOException ex2) {
                if (SeedGenerator.debug != null) {
                    SeedGenerator.debug.println("Failed to use operating system seed generator: " + ex2.toString());
                }
            }
        }
        if (SeedGenerator.instance == null) {
            if (SeedGenerator.debug != null) {
                SeedGenerator.debug.println("Using default threaded seed generator");
            }
            SeedGenerator.instance = new SeedGenerator.ThreadedSeedGenerator();
        }
    }
    
    private static class ThreadedSeedGenerator extends SeedGenerator implements Runnable
    {
        private byte[] pool;
        private int start;
        private int end;
        private int count;
        ThreadGroup seedGroup;
        private static byte[] rndTab;
        
        ThreadedSeedGenerator() {
            this.pool = new byte[20];
            int n = 0;
            this.end = n;
            this.start = n;
            try {
                MessageDigest.getInstance("SHA");
            }
            catch (NoSuchAlgorithmException cause) {
                throw new InternalError("internal error: SHA-1 not available.", cause);
            }
            ThreadGroup[] array = { null };
            Thread thread = (Thread)AccessController.doPrivileged(new PrivilegedAction<Thread>() {
                @Override
                public Thread run() {
                    ThreadGroup threadGroup;
                    ThreadGroup parent;
                    for (threadGroup = Thread.currentThread().getThreadGroup(); (parent = threadGroup.getParent()) != null; threadGroup = parent) {}
                    array[0] = new ThreadGroup(threadGroup, "SeedGenerator ThreadGroup");
                    Thread o = new Thread(array[0], SeedGenerator.ThreadedSeedGenerator.this, "SeedGenerator Thread");
                    o.setPriority(1);
                    o.setDaemon(true);
                    return o;
                }
            });
            this.seedGroup = array[0];
            thread.start();
        }
        
        @Override
        public final void run() {
            while (true) {
                try {
                    synchronized (this) {
                        while (this.count >= this.pool.length) {
                            this.wait();
                        }
                    }
                    int n = 0;
                    int n3;
                    int n4;
                    for (int n2 = n3 = 0; n3 < 64000 && n2 < 6; n = (byte)(n ^ SeedGenerator.ThreadedSeedGenerator.rndTab[n4 % 255]), n3 += n4, ++n2) {
                        try {
                            new Thread(this.seedGroup, new SeedGenerator.ThreadedSeedGenerator.BogusThread(), "SeedGenerator Thread").start();
                        }
                        catch (Exception cause) {
                            throw new InternalError("internal error: SeedGenerator thread creation error.", cause);
                        }
                        for (n4 = 0; System.currentTimeMillis() < System.currentTimeMillis() + 250L; ++n4) {
                            synchronized (this) {}
                        }
                    }
                    synchronized (this) {
                        this.pool[this.end] = n;
                        ++this.end;
                        ++this.count;
                        if (this.end >= this.pool.length) {
                            this.end = 0;
                        }
                        this.notifyAll();
                    }
                    continue;
                }
                catch (Exception cause2) {
                    throw new InternalError("internal error: SeedGenerator thread generated an exception.", cause2);
                }
            }
        }
        
        @Override
        void getSeedBytes(byte[] array) {
            for (int i = 0; i < array.length; ++i) {
                array[i] = this.getSeedByte();
            }
        }
        
        byte getSeedByte() {
            try {
                synchronized (this) {
                    while (this.count <= 0) {
                        this.wait();
                    }
                }
            }
            catch (Exception cause) {
                if (this.count <= 0) {
                    throw new InternalError("internal error: SeedGenerator thread generated an exception.", cause);
                }
            }
            int n;
            synchronized (this) {
                n = this.pool[this.start];
                this.pool[this.start] = 0;
                ++this.start;
                --this.count;
                if (this.start == this.pool.length) {
                    this.start = 0;
                }
                this.notifyAll();
            }
            return n;
        }
        
        static {
            SeedGenerator.ThreadedSeedGenerator.rndTab = new byte[] { 56, 30, -107, -6, -86, 25, -83, 75, -12, -64, 5, -128, 78, 21, 16, 32, 70, -81, 37, -51, -43, -46, -108, 87, 29, 17, -55, 22, -11, -111, -115, 84, -100, 108, -45, -15, -98, 72, -33, -28, 31, -52, -37, -117, -97, -27, 93, -123, 47, 126, -80, -62, -93, -79, 61, -96, -65, -5, -47, -119, 14, 89, 81, -118, -88, 20, 67, -126, -113, 60, -102, 55, 110, 28, 85, 121, 122, -58, 2, 45, 43, 24, -9, 103, -13, 102, -68, -54, -101, -104, 19, 13, -39, -26, -103, 62, 77, 51, 44, 111, 73, 18, -127, -82, 4, -30, 11, -99, -74, 40, -89, 42, -76, -77, -94, -35, -69, 35, 120, 76, 33, -73, -7, 82, -25, -10, 88, 125, -112, 58, 83, 95, 6, 10, 98, -34, 80, 15, -91, 86, -19, 52, -17, 117, 49, -63, 118, -90, 36, -116, -40, -71, 97, -53, -109, -85, 109, -16, -3, 104, -95, 68, 54, 34, 26, 114, -1, 106, -121, 3, 66, 0, 100, -84, 57, 107, 119, -42, 112, -61, 1, 48, 38, 12, -56, -57, 39, -106, -72, 41, 7, 71, -29, -59, -8, -38, 79, -31, 124, -124, 8, 91, 116, 99, -4, 9, -36, -78, 63, -49, -67, -87, 59, 101, -32, 92, 94, 53, -41, 115, -66, -70, -122, 50, -50, -22, -20, -18, -21, 23, -2, -48, 96, 65, -105, 123, -14, -110, 69, -24, -120, -75, 74, 127, -60, 113, 90, -114, 105, 46, 27, -125, -23, -44, 64 };
        }
        
        private static class BogusThread implements Runnable
        {
            @Override
            public final void run() {
                try {
                    for (int i = 0; i < 5; ++i) {
                        Thread.sleep(50L);
                    }
                }
                catch (Exception ex) {}
            }
        }
    }
    
    static class URLSeedGenerator extends SeedGenerator
    {
        private String deviceName;
        private InputStream seedStream;
        
        URLSeedGenerator(String deviceName) throws IOException {
            if (deviceName == null) {
                throw new IOException("No random source specified");
            }
            this.deviceName = deviceName;
            this.init();
            return;
        }
        
        private void init() throws IOException {
            URL url = new URL(this.deviceName);
            try {
                this.seedStream = (InputStream)AccessController.doPrivileged(new PrivilegedExceptionAction<InputStream>() {
                    @Override
                    public InputStream run() throws IOException {
                        if (url.getProtocol().equalsIgnoreCase("file")) {
                            return new FileInputStream(SunEntries.getDeviceFile(url));
                        }
                        return url.openStream();
                    }
                });
            }
            catch (Exception ex) {
                throw new IOException("Failed to open " + this.deviceName, ex.getCause());
            }
        }
        
        @Override
        void getSeedBytes(byte[] b) {
            int length = b.length;
            int i = 0;
            try {
                while (i < length) {
                    int read = this.seedStream.read(b, i, length - i);
                    if (read < 0) {
                        throw new InternalError("URLSeedGenerator " + this.deviceName + " reached end of file");
                    }
                    i += read;
                }
            }
            catch (IOException cause) {
                throw new InternalError("URLSeedGenerator " + this.deviceName + " generated exception: " + cause.getMessage(), cause);
            }
        }
    }
}
